require 'puppet/util/network_device'
require 'puppet/util/network_device/equallogic/facts'
require 'puppet/util/network_device/transport_equallogic'
require 'uri'
require 'net/https'
require 'security/encode'

class Puppet::Util::NetworkDevice::Equallogic::Device

  attr_accessor :url, :transport

  $provider_path = Pathname.new(__FILE__).parent
  $pythonFilePath = "#{$provider_path}/pythonEquallogic.py"
  def initialize(url, option = {})
    Puppet.debug("Device login started")
    @url = URI.parse(url)

    @transport = Puppet::Util::NetworkDevice::Transport_equallogic.new
    @transport.host = @url.host
    @transport.user = URI.decode(@url.user) if @url.user
    @transport.password = URI.decode(@url.password) if @url.password
    @transport.password = URI.decode(asm_decrypt(@transport.password)) if @transport.password

    params = CGI.parse(@url.query || '')

    # Get SNMP community string from query arguments; default to public
    if params['community_string'].empty?
      @transport.snmpcommunity = "public"
    else
      @transport.snmpcommunity = params['community_string'].first
    end

    # Optionally use ASM credential_id. Overrides all other credentials
    unless params['credential_id'].empty?
      require 'asm/cipher'
      cred = ASM::Cipher.decrypt_credential(params['credential_id'].first)
      @transport.user = cred.username
      @transport.password = cred.password
      @transport.snmpcommunity = cred.snmp_community_string
    end

    raise ArgumentError, "Invalid scheme #{@url.scheme}. Must be https" unless @url.scheme == 'https'
    raise ArgumentError, "no user specified" unless @transport.user
    raise ArgumentError, "no password specified" unless @transport.password

    libPath = "#{$provider_path}/lib"
    ENV['LD_LIBRARY_PATH'] = "#{libPath}"
    ENV['PS_GROUP_ADDR'] = "#{@transport.host}"
    ENV['PS_GROUP_USER'] = "#{@transport.user}"
    ENV['PS_GROUP_PASSWORD'] = "#{@transport.password}"
    response = checkConnection
    if (response.match("Error"))
      raise Puppet::Error, "Unable to set up a connection with the equallogic device because of the following exception: #{response}."
    end
  end

  def checkConnection
    response = `python #{$pythonFilePath} #{@transport.host} #{@transport.user} #{@transport.password} show`
    return response
  end

  def parse_snmp(query)
    if query
      params = CGI.parse(query)
      params['community_string'].first unless params['community_string'].empty?
    end
  end

  def facts
    @facts ||= Puppet::Util::NetworkDevice::Equallogic::Facts.new(@transport)
    facts = @facts.retrieve

    facts
  end
end
